/**
 * @file media.h Gaim Media API
 * @ingroup core
 *
 * gaim
 *
 * Copyright (C) 2003, Tim Ringenbach <omarvo@hotmail.com>
 * Copyright (C) 2005, Bleeter Yaluser <bleeter from bleeter.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _GAIM_MEDIA_H_
#define _GAIM_MEDIA_H_

#include <gst/gst.h>
#include "connection.h"

/**************************************************************************/
/** Data Structures                                                       */
/**************************************************************************/

typedef struct _GaimMedia GaimMedia;

/**
 * Everything the core needs to know about media it's downloading.
 */
struct _GaimMedia {
	GaimConnection *gc; /**< The GaimConnection associated with the media. */
	gchar *name;        /**< The name of the buddy who's media we're connected to. */
	GstElement *sink;  /**< The element with a sink pad where stuff is going to */
	GstElement *src;   /**< An element with a src pad */
	GstElement *pipeline; /**< The pipeline */
	gpointer ui_data;   /**< A place for the ui to stick its extra data. */
	gpointer proto_data;/**< A place for the prpl to stick its extra data. */
};

/**
 * The reason gaim_media_got_close() was called.
 */
enum gaim_media_close_reason {
	MEDIA_UNKNOWN = 0, /**< Unknown reason. */
	MEDIA_STOPPED_CASTING, /**< They turned the media off. */
	MEDIA_PERM_CANC, /**< They canceled permission. */
	MEDIA_USER_DECL, /**< They declined our attempt to view them. */
	MEDIA_NOT_ONLINE, /**< The media isn't casting. */
	MEDIA_CONNECTION_PEER_FAILED, /**< The direct connection failed. */
	MEDIA_CONNECTION_SERVER_FAILED, /**< Connectioning to some centeralized server failed */
};

/**
 * The UI Ops structure.
 *
 * The UI fills this out with functions it wants called when media related stuff happens.
 */
struct gaim_media_ui_ops
{
	void (*new)(GaimMedia *gwc); /**< Core calls this when a new media window needs opening. */
	void (*close_cam)(GaimMedia *gwc);     /**< The core will call this to close the media.*/
	void (*got_invite)(GaimConnection *gc, const gchar *who); /**< Call when an invite is received. */
};

#ifdef __cplusplus
extern "C" {
#endif

/**************************************************************************/
/** @name Media Viewing (downloading/receiving) API                      */
/**************************************************************************/
/*@{*/

/**
 * Creates a new media.
 *
 * Prpl's call this when they'll have media data soon.
 * The core then tells the UI to pop up a window or something
 * and get ready.
 *
 * @param gc The GaimConnection associated with the media.
 * @param from The screenname of the buddy who's media we are about to connect to.
 * @return A newly allocated GaimMedia pointer, to be used as a handle
 *         for most other functions from here on out.
 */
GaimMedia *gaim_media_new(GaimConnection *gc, const char *from);

void gaim_media_set_sink(GaimMedia *wc, GstElement *sink);
void gaim_media_set_src(GaimMedia *wc, GstElement *src);
/**
 * Closes and destroys open media.
 *
 * The UI calls this when the user tells it to close the media.
 *
 * @param gwc The media to close. It should not be used after this call,
 *            as it will be g_free()'d.
 */
void gaim_media_close(GaimMedia *gwc);

/**
 * Called when we get invited to connect to media
 *
 * The prpl calls this function when it receives a
 * media invitation.
 *
 * @param gc The GaimConnection associated with the invite.
 * @param from The buddy who invited us.
 */
void gaim_media_got_invite(GaimConnection *gc, const char *from);

/**
 * The media was closed by the other end.
 *
 * The prpl calls this is the media gets closed from the other end.
 *
 * @param gwc The media handle. Will be g_free()'d after this call.
 * @param reason The reason for the closure.
 */
void gaim_media_got_close(GaimMedia *gwc, enum gaim_media_close_reason reason);

/**
 * Called by UI when we accepted an invite.
 *
 * @param gc The GaimConnection.
 * @param who The person that invited us.
 */
void gaim_media_invite_accept(GaimConnection *gc, const gchar *who);

/**
 * Called by UI when we declined an invite.
 *
 * @param gc The GaimConnection.
 * @param who The person that invited us.
 */
void gaim_media_invite_decline(GaimConnection *gc, const gchar *who);

/*@}*/

/**************************************************************************/
/** @name Media Sending (casting/uploading) API                          */
/**************************************************************************/
/*@{*/
// This API needs major reworking
/**
 * Called by the prpl when we're casting and someone connects,
 * disconnects, or requests to connect.
 *
 * @param gc The GaimConnection.
 * @param who The person who is or wants to view our media.
 * @param connect 0 = disconnect, 1=connect, 2=request. FIXME: change this to an enum.
 */
void gaim_media_viewer(GaimConnection *gc, char *who, int connect);

/**
 * We invited someone to view our media and they accepted.
 *
 * @param gc The GaimConnection associated with invite.
 * @param from The person who accepted our invitation.
 */
void gaim_media_got_invite_accept(GaimConnection *gc, const char *from);

/**
 * We invited someone to vie our media, but they declined.
 *
 * @param gc The GaimConnection.
 * @param from The person who declined our invite.
 */
void gaim_media_got_invite_decline(GaimConnection *gc, const char *from);

/*@}*/

/**************************************************************************/
/** @name UI Registration Functions                                       */
/**************************************************************************/
/*@{*/

/**
 * Sets the UI operations structure to be used for the medias.
 *
 * @param ops The ops struct.
 */
void gaim_media_set_ui_ops(struct gaim_media_ui_ops *ops);

/**
 * Returns the UI operations structure to be used for the medias.
 *
 * @return The UI operations structure.
 */
struct gaim_media_ui_ops *gaim_media_get_ui_ops(void);

/*@}*/

#ifdef __cplusplus
extern "C" {
#endif
#endif /* _GAIM_MEDIA_H_ */
